wren
====

Сервис обработки запросов на получение данных внутри заданных временных периодов. От простого запроса напрямую к БД метрик
он отличается возможностью дополнить запрошенные отсчеты данных контекстной информацией, позволяющей понятнее
отобразить и осмыслить полученную информацию.

Зачем это нужно? Давайте рассмотрим пример:

> sdb2.disk.free.bytes:12345600@ms

В этой метрики содержится размер свободного места на одном конкретном разделе. Несмотря на кажующуюся полноту, этот срез
данных не отвечает на важный вопрос - сколько _занято_ места на этом разделе? Без адекватной точки отсчета сложно оценить
общую заполненность места и предпринять какие-то меры. 

Первое, что приходит в голову - отсылать параллельно с количеством свободного места на диске и количество занятого:

> _sdb2.disk.free.bytes:12345600@ms_

> _sdb2.disk.taken.bytes:87654400@ms_

Но давайте посмотрим на следующую формулу:
> _sdb2.disk.free.bytes_ + _sdb2.disk.taken.bytes_ = _12345600_ + _87654400_ = _10000000_

В большинстве случаев, общий объем имеющегося места на диске всегда будет равен какому-то определенному числу, и передавать вдобавок к количеству свободных байт информацию будет избыточным - их сумма всегда будет равна общему количеству места на диске. Размер диска меняется крайне редко, поэтому достаточно сохранить эту информацию один раз и впоследствии обращаться к ней при необходимости отобразить временные отсчеты в понятном контексте.

При чем же тут wren?
--------------------

**wren** выступает как прокси между БД с временными отсчетами и клиентским приложением, дополняя по мере надобности полученные
значения контекстными метаданными. Также он выступает как интерфейс к хранилищу контекстных данных, позволяя сохранять их и просматривать их содержимое. 

Основные понятия
----------------

**Метрика**

Временной ряд, значения которого является измерениями определенной величины в указанные промежутки времени. Эти промежутки могут быть как постоянными (количество свободной памяти, загруженность CPU и т.д.), так и произвольными (заведение нового пользователя, приход запроса и т.д.).

Пример:

> v1.tatar.bars.wren.instance_test.parameter1_value.parameter2_value2.metric 1 1234567890

При проектировании системы мониторинга было предложено использовать следующий формат кодирования имен метрик:

> version.region.client.product.instance_name.value 1 1234567890

* _v1_ - версия протокола;
* _region_ - код региона;
* _client_ - код клиента;
* _product_ - код продукта;
* _metric_ - название метрики.

Эти поля являются обязательными для имени метрики, и компоненты системы будут регистрировать её значение только в случае соответствия формата. При этом возможно указание _необязательных_ параметров, которые описываются в следующем виде:

> v1.region.client.product. **имя_значение** .value 1 1234567890

В имени метрики может встречаться один или более необязательный параметр.

**Контекст**

Набор метаданных, используемых для придания смысла численным данным собранных метрик. Контекст представляет собой обыкновенный
словарь с хэшами кодовыми значениями описываемых объектов в качестве ключей и словарями с информацией об объектах в значениях. 

Хэширование необходимо для помещения ссылки на описанные объекты в имена метрик, которые имеют ограниченные наборы символов ([A-Za-z0-9\._]).

Пример:
<pre>
{
    "36145dffb81b3b85662c4408d7b05b14": {
        "disk": "sda5"
    }, 
    "7f54f26f1d1c4e554e6da9773c17aff0": {
        "disk": "sda2"
    }, 
    "cccb3c12390226afffa541934c486c10": {
        "disk": "sdb1"
    }, 
    "dfae47a303fedb19d410e3756439f4dc": {
        "disk": "sda1"
    }, 
    "e04d668cafae4aa08ada17a085c8ba64": {
        "disk": "sdb2"
    }
}
</pre>

Приведенный пример описывает имеющиеся системные разделы, и ключами его являются MD5-хэши соответствующих имен разделов (e.g. MD5("sdb1") => "cccb3c12390226afffa541934c486c10").

Соответственно, в имени метрики это будет отображено следующим образом:
> v1.bars.tatar.test.instance_test. **disk_cccb3c12390226afffa541934c486c10** .io.read.bytes_per_sec


**Идентификация**

Набор из 4-х строковых значений, однозначно определяющих принадлежность контекста или метрики
к определенному экземпляру продукта, запущенного на обозначенном сервере.

Для контекстов он задается следующей структурой данных: 

<pre>
"identity": {
    "region": "testregion",
    "client": "testclient",
    "product": "testsoftware",
    "instance": "testserver"
}
</pre>, где

* **region** - кодовое обозначение региона, в котором развернут программный продукт (пример: _tatar_, _kchr_, _nsk_),
* **client** - -//- компании, эксплуатирующей ПП;
* **product** - -//- обозначение ПП;
* **instance** - физический или виртуальный сервер, на котором развернут ПП.

Установка
=========

Все просто:

> pip install wren -i http://pypi.bars-open.ru/simple

Настройка
=========

Конфигурация системы прописывается в файле _wren.yaml_.

Секции:

* **bound_contexts** - словарь, описывающий привязку конкретной метрики к конкретному контексту. Если будет запрошена одна
из указанных в ключах словаря метрик, то к ней будут присоединены соотвествующие контекстные данные. 
* **kairosdb** - словарь, описывающий параметры подключения к REST API БД хранения метрик KairosDB.
 * **host** - доменное имя сервера с REST API;
 * **port** - порт (по умолчаниию - _8080_).
* **redis** - словарь, описывающий параметры подключения к Redis.
 * **host** - доменное имя сервера, на котором установлен Redis;
 * **port** - порт (по умолчаниию - _6379_).

Пример файла конфигурации:

<pre>
bound_contexts:
    disk.used.bytes:
        storage: mount

kairosdb:
    host: '91.203.194.148'
    port: 8080

redis:
    host: '91.203.194.148'
    port: 6379
</pre>

Файл конфигурации ищется в папке, где был запущен процесс.